package de.catma.util;

/**
 * Check digit algorithm. The DAMM Algorithm implementation. 
 * It detects all single-digit errors and all transposition errors.
 * 
 * @author db
 *
 */
public class DammAlgorithm {

	/**
	 * quasigroup as defined and prepared in paper.
	 */
	 private static int[][] qg = new int[][] {
	        { 0, 3, 1, 7, 5, 9, 8, 6, 4, 2 }, 
	        { 7, 0, 9, 2, 1, 5, 4, 8, 6, 3 }, 
	        { 4, 2, 0, 6, 8, 7, 1, 3, 5, 9 },
	        { 1, 7, 5, 0, 9, 8, 3, 4, 2, 6 }, 
	        { 6, 1, 2, 3, 0, 4, 5, 9, 7, 8 }, 
	        { 3, 6, 7, 4, 2, 0, 9, 5, 8, 1 }, 
	        { 5, 8, 6, 9, 7, 2, 0, 1, 3, 4 },
	        { 8, 9, 4, 5, 3, 6, 2, 0, 1, 7 },
	        { 9, 4, 3, 8, 6, 1, 7, 2, 0, 5 }, 
	        { 2, 5, 8, 1, 4, 3, 6, 7, 9, 0 } 
	        };

    public static int calculateCheckSumDigit(int number) {
        return String.valueOf(number).chars()
        	.sequential()
        	.map(c -> c - 48) //map ascii number to original one
        	.reduce(0, (d,acc) -> qg[acc][d]); //do the calculation
         }
    
    public static boolean validate(int number){
    	return calculateCheckSumDigit(number) == 0;
    }
    
    public static int padChecksum(int number){
    	return number * 10 + calculateCheckSumDigit(number);
    }
}
